import java.util.Arrays;
import java.util.Random;


public class MainBuggyMTMergeSort {

	/**
	 * Sehr viele Threads sind viel schneller, als einige wenige. Grund dafr
	 * ist die ineffiziente Implementation der Methode merge in
	 * BuggySeqMergeSort. Dort wird fr jeden Merge-Vorgang, egal wie klein die
	 * zu mergenden Arrays auch sind, ein neues Array der kompletten Grsse
	 * erstellt. Das stndige allozieren und aufrumen den bentigten
	 * Speicherplatzes verlangsamt das Sortieren enorm.
	 * Wenn alle parallen Sortiererthreads fertig sind, wird ihr Ergebnis mit der
	 * effizienteren merge-Methode in BuggyMTMergeSort zusammengemerged.
	 * Als Resultat luft das Programm am schnellsten, wenn es ber 1000 Threads
	 * zur Verfgung hat.
	 */
	public static void main(String[] args) {
		int no_elements = 100007;
		int no_threads = 119;
		int[] int_to_sort;
		int[] orig_elements;

		// create an int array of size no_elements and create a copy
		// of that array

		int_to_sort = createRandIntArray(no_elements);
		orig_elements = new int[no_elements];
		System.arraycopy(int_to_sort,0,orig_elements,0,int_to_sort.length);

		//constructor for the class to perform multithreaded merge sort
		BuggyMTMergeSort m = new BuggyMTMergeSort(int_to_sort, no_threads);
		long t0 = System.nanoTime();
		m.sort();
		long dt = System.nanoTime()-t0;
		System.out.println("Time elapsed: "+dt/1000000+" ms");

		//check if your implementation is correct
		check(orig_elements,int_to_sort);
	}

	 /*
	   * This function creates an array of size no_elements and
	   * initializes it with random integer numbers
	   */
	  private static int[] createRandIntArray(int no_elements) {
		  int i;
		  int[] tmp = new int[no_elements];
		  Random random = new Random();

		  for(i = 0; i < tmp.length; i++) {
			  tmp[i] = random.nextInt();
		  }

		  return tmp;
	  }


	  /*
	   * This function checks the mergesort results against a sorting
	   * algorithm privided by the Java API
	   */
	  private static void check(int[] orig_array, int[] merge_array) {
		  Arrays.sort(orig_array);

		  if(Arrays.equals(orig_array, merge_array)) {
			    System.out.print("SUCCESS!\n");
		  }
		  else {
			  System.out.print("FAILURE!\n");
		  }
	  }


}
